@@ -28,13 +28,17 @@ class OrthancCloner(OrthancMonitor):
                  mode: ClonerMode = ClonerMode.DEFAULT,
                  destination_peer: str = None,                    # the 'alias' of the destination peer if declared in Orthanc.  It must be defined for PEERING and TRANSFER mode
                  scheduler: Scheduler = None,
-                 ):
+                 max_retries: int = 5,
+                 error_folder_path: str = None
+        ):
         super().__init__(
             api_client=source,
             worker_threads_count=worker_threads_count,
             persist_status_path=persist_status_path,
             polling_interval=polling_interval,
-            scheduler=scheduler
+            scheduler=scheduler,
+            max_retries=max_retries,
+            error_folder_path=error_folder_path
         )
 
         self._destination = destination
@@ -56,26 +60,24 @@ class OrthancCloner(OrthancMonitor):
         if self._mode in [ClonerMode.PEERING, ClonerMode.DEFAULT]:
             self.add_handler(ChangeType.NEW_INSTANCE, self.handle_new_instance)
         elif self._mode == ClonerMode.TRANSFER:
-            self.add_handler(ChangeType.NEW_STUDY, self.handle_new_study)
+            self.add_handler(ChangeType.STABLE_STUDY, self.handle_stable_study)
 
 
-    def handle_new_instance(self, instance_id, api_client):
+    def handle_new_instance(self, change_id, instance_id, api_client):
         try:
             if self._mode == ClonerMode.DEFAULT:
                 dicom = api_client.instances.get_file(instance_id)
 
                 self._destination.upload(dicom)
-                logger.info(f"copied instance {instance_id}")
-                return True
+                logger.info(f"{change_id}, copied instance {instance_id}")
             elif self._mode == ClonerMode.PEERING:
                 api_client.peers.send(target_peer=self._destination_peer, resources_ids=instance_id)
 
         except Exception as ex:
-            logger.error(f"Error while cloning instance {instance_id}: {str(ex)}")
-            return False
+            raise Exception(f"Error while cloning instance {instance_id}: {str(ex)}")
 
 
-    def handle_new_study(self, study_id, api_client):
+    def handle_stable_study(self, change_id, study_id, api_client):
         try:
             if self._mode == ClonerMode.TRANSFER:
                 transfer_job = api_client.transfers.send(
@@ -85,12 +87,10 @@ class OrthancCloner(OrthancMonitor):
                 )
                 transfer_job.wait_completed(timeout=None)
 
-            logger.info(f"transfered study {study_id}")
-            return True
+            logger.info(f"{change_id} transfered study {study_id}")
 
         except Exception as ex:
-            logger.error(f"Error while transferring study {study_id}: {str(ex)}")
-            return False
+            raise Exception(f"Error while transferring study {study_id}: {str(ex)}")
 
 # examples:
 # python orthanc_tools/orthanc_cloner.py --source_url=http://192.168.0.10:8042 --source_user=user --source_pwd=pwd --dest_url=http://192.168.0.10:8042 --dest_user=user --dest_pwd=pwd
@@ -113,8 +113,10 @@ if __name__ == '__main__':
     parser.add_argument('--dest_pwd', type=str, default=None, help='Orthanc destination password')
     parser.add_argument('--dest_peer', type=str, default=None, help='Orthanc destination peer (peer alias in source Orthanc)')
     parser.add_argument('--mode', type=str, default=None, help='Cloner Mode (Default, Peering, Transfer)')
-    parser.add_argument('--persist_state_path', type=str, default=None, help='Path where the state of the cloner will be saved (to resume later)')
+    parser.add_argument('--persist_state_path', type=str, default=None, help='File path where the state of the cloner will be saved (to resume later)')
     parser.add_argument('--worker_threads_count', type=int, default=1, help='Number of worker threads')
+    parser.add_argument('--error_folder_path', type=str, default=None, help='Folder path where to store error reports')
+    parser.add_argument('--max_retries', type=int, default=5, help='Number of retries in case of error')
 
     Scheduler.add_parser_arguments(parser)
 
@@ -129,7 +131,9 @@ if __name__ == '__main__':
     dest_peer = os.environ.get("DEST_PEER", args.dest_peer)
     mode = os.environ.get("MODE", args.mode)
     persist_state_path = os.environ.get("PERSIST_STATE_PATH", args.persist_state_path)
-    worker_threads_count = int(os.environ.get("WORKERS_THREADS_COUNT", str(args.worker_threads_count)))
+    worker_threads_count = int(os.environ.get("WORKER_THREADS_COUNT", str(args.worker_threads_count)))
+    error_folder_path = os.environ.get("ERROR_FOLDER_PATH", args.error_folder_path)
+    max_retries = os.environ.get("MAX_RETRIES", args.max_retries)
 
     scheduler = Scheduler.create_from_args_and_env_var(args)
 
@@ -144,7 +148,9 @@ if __name__ == '__main__':
         mode=mode,
         destination_peer=dest_peer,
         scheduler=scheduler,
-        worker_threads_count=worker_threads_count
+        worker_threads_count=worker_threads_count,
+        error_folder_path=error_folder_path,
+        max_retries=max_retries
     )
 
     cloner.execute(existing_changes_only=False)
